/*
 * Copyright (C) 2015 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License
 */

package tw.qtlin.mac.airunlocker;

import android.Manifest;
import android.app.Activity;
import android.app.ActivityManager;
import android.app.KeyguardManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.hardware.fingerprint.FingerprintManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;



import javax.crypto.Cipher;
import javax.inject.Inject;

/**
 * Main entry point for the sample, showing a backpack and "Purchase" button.
 */





public class MainActivity extends Activity {
    // The request code
    static final int REQUEST_SCAN_QRCODE = 1;
    static final int REQUEST_ENABLE_BT = 2;
    private final int PERRMISSION_REQUEST = 3;

    private static final int CONNECT_SUCCESS = 88;
    private static final int CONNECT_FAILE = 89;
    private static final int CONNECT_DEVICE_NOT_FOUND = 90;

    private static final String KEY_NAME = "my_key";
    private static final String TAG = MainActivity.class.getSimpleName();

    private static final String DIALOG_FRAGMENT_TAG = "myFragment";


    private Button mButtonPair;




    private boolean isMyServiceRunning(Class<?> serviceClass) {
        ActivityManager manager = (ActivityManager) getSystemService(Context.ACTIVITY_SERVICE);
        for (ActivityManager.RunningServiceInfo service : manager.getRunningServices(Integer.MAX_VALUE)) {
            if (serviceClass.getName().equals(service.service.getClassName())) {
                return true;
            }
        }
        return false;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);


        setContentView(R.layout.activity_main);

        mButtonPair = (Button)findViewById(R.id.button_pair);
        mButtonPair.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent(MainActivity.this, SimpleScannerActivity.class);
                startActivityForResult(intent, REQUEST_SCAN_QRCODE);
                finish();
                startActivity(getIntent());
            }
        });







//        if (!mKeyguardManager.isKeyguardSecure()) {
//            // Show a message that the user hasn't set up a fingerprint or lock screen.
//            Toast.makeText(this,
//                    "Secure lock screen hasn't set up.\n"
//                            + "Go to 'Settings -> Security -> Fingerprint' to set up a fingerprint",
//                    Toast.LENGTH_LONG).show();
//            unlockButton.setEnabled(false);
//            Intent i = new Intent(DevicePolicyManager.ACTION_SET_NEW_PASSWORD);
//            startActivity(i);
//            return;
//        }

//        //noinspection ResourceType
//        if (!mFingerprintManager.hasEnrolledFingerprints()) {
//            unlockButton.setEnabled(false);
//            // This happens when no fingerprints are registered.
//            Toast.makeText(this,
//                    "Go to 'Settings -> Security -> Fingerprint' and register at least one fingerprint",
//                    Toast.LENGTH_LONG).show();
//            return;
//        }


    }



    @Override
    protected void onResume() {
        super.onResume();
        Log.i(TAG,"onResume");
        if (ContextCompat.checkSelfPermission(this,
                Manifest.permission.ACCESS_FINE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {


            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.ACCESS_FINE_LOCATION},
                    PERRMISSION_REQUEST);
            return;

        }
        if (ContextCompat.checkSelfPermission(this,
                Manifest.permission.ACCESS_COARSE_LOCATION)
                != PackageManager.PERMISSION_GRANTED) {


            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.ACCESS_COARSE_LOCATION},
                    PERRMISSION_REQUEST);
            return;

        }

        Intent ServiceIntent = new Intent(this, BLEService.class);
        if(!isMyServiceRunning(BLEService.class)){
            Log.i(TAG, "Start service from MainActivity");
            startService(ServiceIntent);

        }






    }



    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int id = item.getItemId();

        if (id == R.id.action_settings) {
            Intent intent = new Intent(this, SettingsActivity.class);
            startActivity(intent);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }



    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if(requestCode == PERRMISSION_REQUEST && resultCode != RESULT_OK)
            finish();
    }
}
